<?php

/**
 * @file
 * Functions for the oEmbed filter
 */

function _oembed_filter_apply($text, $filter, $format, $langcode, $cache, $cache_id) {
  global $_oembed_default_attributes;
  $_oembed_default_attributes = array_filter(array(
    'maxwidth' => intval($filter->settings['oembed_maxwidth']),
    'maxheight' => intval($filter->settings['oembed_maxheight']),
  ));
  $text = preg_replace_callback("`(^|<p(?:\s[^>]*)*>|<li(?:\s[^>]*)*>|<br(?:\s[^>]*)*>|[ \n\r\t\(])((http://|https://|ftp://|mailto:|smb://|afp://|file://|gopher://|news://|ssl://|sslv2://|sslv3://|tls://|tcp://|udp://)([a-zA-Z0-9@:%_+*~#?&=.,/;-]*[a-zA-Z0-9@:%_+*~#&=/;-]))([.,?!]*?)(?=($|</p>|</li>|<br\s*/?>|[ \n\r\t\)]))`i", '_oembed_preg_parse', $text);
  unset($_oembed_default_attributes);
  return $text;
}

/**
 * Settings callback for the oEmbed filter.
 */
function _oembed_filter_settings($form, &$form_state, $filter, $format, $defaults) {
  $filter->settings += $defaults;

  $settings = array();
  $settings['oembed_maxwidth'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum width of embed'),
    '#default_value' => $filter->settings['oembed_maxwidth'],
    '#description' => t('The maximum width of an embed, isn\'t respected by all providers'),
  );
  $settings['oembed_maxheight'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum height of embed'),
    '#default_value' => $filter->settings['oembed_maxheight'],
    '#description' => t('The maximum height of an embed, isn\'t respected by all providers'),
  );
  return $settings;
}

function _oembed_preg_parse($match) {
  return _oembed_resolve_link($match[2], $match[1], $match[5]);
}

function _oembed_resolve_link($match, $prefix, $suffix) {
  global $_oembed_default_attributes;

  $return = '';
  $url = decode_entities($match);
  $matches = array();
  $element = array();
  if ($provider = oembedcore_get_provider($url, $matches)) {
    $element = oembedcore_render_cache('oembed_bare', $url, $_oembed_default_attributes, REQUEST_TIME + 600);

    // In filters, theme must have no impact on oembed output, so the cache key
    // is recalculated to exclude theme. This is normally done in
    // drupal_render_cache_set(), but it's OK here because the array is rendered
    // immediately.
    $cid_parts = drupal_render_cid_parts($element['#cache']['granularity']);
    unset($cid_parts[0]);

    $element['#cache']['cid'] = implode(':', array_merge($element['#cache']['keys'], $cid_parts));
    $return = drupal_render($element);
  }
  if (empty($return)) {
    $return = $url;
  }

  return $prefix . $return . $suffix;
}
